c
c Supplementary Material, Software, for the Textbook:
c
c Mudelsee M (2020) Statistical Analysis of Climate Extremes,
c                   Cambridge University Press, 200 pp,
c                   ISBN 978-1-107-03318-4 (Hardback),
c                   ISBN 978-1-108-79146-5 (Paperback),
c                   https://www.manfredmudelsee.com/textbook
c
c=======================================================================
c
c GEV distribution, ML estimation of parameters
c
c ML routine after Hosking (1985), Macleod (1989)
c
c See subroutine MLEGEV.
c
c FORTRAN 77 driver program by Mudelsee (search for 'Mudelsee')
c =======================================================================
c
c (1) notation of GEV parameters,
c (2) standard errors output,
c (3) return level output.
c
c=======================================================================
c
c
c Copyright notice:     Copyright (C)
c                       Climate Risk Analysis - Manfred Mudelsee
c                       https://www.climate-risk-analysis.com
c
c Permission notice:    Permission is hereby granted, free of charge,
c                       to any person obtaining a copy of this software
c                       and associated computer files -- excluding the
c                       ML routine (subroutine MLEGEV) after
c                       Hosking (1985) and Macleod (1989) --
c                       (the "Software"),
c                       to deal in the Software without restriction,
c                       including without limitation the rights to use,
c                       copy, modify, merge, publish, distribute,
c                       sublicense, and/or sell copies of the Software,
c                       and to permit persons to whom the Software is
c                       furnished to do so, subject to the following
c                       conditions:
c
c                       The above copyright notice, including the URL
c                       https://www.climate-risk-analysis.com,
c                       and this permission notice shall be included
c                       in all copies or substantial portions of the
c                       Software.
c
c=======================================================================
c
c Disclaimer:           THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
c                       WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
c                       INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
c                       MERCHANTABILITY, FITNESS FOR A PARTICULAR
c                       PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
c                       THE AUTHOR OR COPYRIGHT HOLDER BE LIABLE FOR ANY
c                       CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
c                       ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
c                       FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
c                       OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
c
c=======================================================================
c
c Developer:            Climate Risk Analysis - Manfred Mudelsee
c                       Kreuzstrasse 27
c                       Heckenbeck
c                       37581 Bad Gandersheim
c                       Germany
c
c Contact:              http://www.climate-risk-analysis.com
c                       mudelsee@climate-risk-analysis.com
c
c=======================================================================
c
c Fortran 77 Compilation
c ======================
c
c gfortran compiler command: gfortran -o gevmlest.exe -O2 gevmlest.for
c
c=======================================================================
c
c
c References
c ==========
c
c Coles (2001), Hosking (1985), Lu and Stedinger (1992),
c Macleod (1989).
c
c For the bibliographic details, see
c Mudelsee (2014) Climate Time Series Analysis: Classical and
c                 Bootstrap Methods, 2nd edition. Springer, Cham,
c                 Switzerland, 454 pp.
c                 [excerpt, including reference list, available at
c                 http://www.manfredmudelsee.com/book]
c
      implicit none
      character(len=80) infile       ! input data, t(1:m), x(1:m)
      integer mmax                   ! maximum data size
      parameter (mmax=1000000)
      double precision t             ! time (dummy)
      double precision x(mmax)       ! data
      integer m                      ! data size
      integer monit                  ! flag for monitoring iterations
      parameter (monit=1)
      integer ifault                 ! failure indicator
      integer i
      integer npara                  ! number of parameters
      parameter (npara=3)
      double precision para(npara)   ! GEV parameters
      double precision mu            ! re-written, GEV location
      double precision sigma         ! re-written, GEV scale
      double precision xi            ! re-written, GEV shape
      double precision vcov(2*npara) ! COV matrix
      double precision xp            ! return level
      double precision p             ! tail probability
      double precision q             ! 1-p
      double precision varxp         ! parameters for xp 
      double precision varmu         ! error determination
      double precision varsigma      ! "
      double precision varxi         ! "
      double precision covmusigma    ! "
      double precision covmuxi       ! "
      double precision covsigmaxi    ! "
      double precision dxdmu         ! "
      double precision dxdsigma      ! "
      double precision dxdxi         ! "
      external mlegev
c
c 1.  Initial stuff
c     =============
c
      print *
      print *
      print *
      print *, ' GEVMLEST'
      print *, ' ========'
      print *
      print *, ' Maximum likelihood estimation of GEV distribution'
      print *, ' on time series data'
      print *
      print *
      print *
      print *, ' GEVMLEST: Textbook'
      print *, ' =================='
      print *
      print *,
     +' Mudelsee M (2020) Statistical Analysis of Climate Extremes.'
      print *,
     +' Cambridge University Press.'
      print *,
     +' http://www.climate-risk-analysis.com'
      print *
      print *
      print *
      print *, ' GEVMLEST: Original source of routine'
      print *, ' ===================================='
      print *, ' Hosking (1985) Applied Statistics 34(3):301-310.'
      print *
      print *
      print *
      print *,' Input data file name (time series):'
      read (5,*) infile
      open (unit=1, file=infile, form='formatted', status='old')
c      
      print '(a,i7,a)', '  Input data size (maximum: ',mmax,'):'
      read (5,*) m
c
      print *,' Pre-define tail probability:'
      read (5,*) p
c
      do i=1,mmax
         x(i)=0.0
      end do
      do i=1,m
         read (1,*) t,x(i)
      end do
      close (unit=1, status='keep')
c
      print *,' Input initial estimate location parameter (mu): '
      read (5,*) para(1)
c
      print *,' Input initial estimate scale parameter (sigma): '
      read (5,*) para(2)
c
      print *,' Input initial estimate shape parameter (xi): '
      read (5,*) para(3)
      para(3)=-para(3)
c
c 2.  Estimation of GEV parameters
c     ============================
c
      print *
      print *
      print *
      print *,
     +' GEVMLEST numerically minimizes the log-likelihood function'
      print *,
     +' =========================================================='
      print *
      print *,' See Hosking (1985) for explanations'
      print *
      print *
      print *
      call MLEGEV(x,m,para,vcov,monit,ifault)
c
c 3.  Calculation of return period and standard errors
c     ================================================
c
      mu=para(1)
      sigma=para(2)
      xi=-para(3)
      q=1.0-p
      xp=mu-(sigma/xi)*(1.0-(-log(q))**(-xi))
      dxdmu=1.0
      dxdsigma=(-1.0/sigma)*(1.0-(-log(q))**(-xi))
      dxdxi=(sigma/xi**2.0)*(1.0-(-log(q))**(-xi))-
     +        (sigma/xi)*(-log(q))**(-xi)*
     +        log(-log(q))
      varmu=vcov(1)
      varsigma=vcov(3)
      varxi=vcov(6)
      covmusigma=vcov(2)
      covmuxi=vcov(4)
      covsigmaxi=vcov(5)
      varxp=dxdmu**2.0*varmu+dxdsigma**2.0*varsigma+dxdxi**2.0*varxi+
     +        2.0*dxdmu*dxdsigma*covmusigma+
     +        2.0*dxdmu*dxdxi*covmuxi+
     +        2.0*dxdsigma*dxdxi*covsigmaxi
c
c 4.  Result
c     ======
c
      print *
      print *
      print *
      print *, ' GEVMLEST: Results (estimates and standard errors)'
      print *, ' ================================================='
      print *
      print *, ' Data file  name (time series):           ',infile
      print *, ' Data size:                               ',m
      print *,
     +      '  Pre-defined tail probability:            ',p
      print *
      print *,
     +      '  Estimated location parameter       mu  = ',mu
      print *,
     +      '                                  se(mu) = ',sqrt(varmu)
      print *
      print *,
     +      '  Estimated scale parameter       sigma  = ',sigma
      print *,
     +      '                               se(sigma) = ',sqrt(varsigma)
      print *
      print *,
     +      '  Estimated shape parameter          xi  = ',xi
      print *,
     +      '                                  se(xi) = ',sqrt(varxi)
      print *
      print *,
     +      '  Derived return level               xp  = ',xp
      print *,
     +      '                                  se(xp) = ',sqrt(varxp)
      print *
      print *
      print *
      print '(a)',
     +      'GEVMLEST results for spreadsheet programs input data sheet'
      print *
      print
     + '(a,1x,a,1x,a,1x,a,1x,a,1x,a,1x,a,1x,a,1x,a,1x,a,1x,a,1x,a)',
     +      '                 infile',
     +      '    m',
     +      '    p',
     +      '      1/p',
     +      '       mu',
     +      '   se(mu)',
     +      '    sigma',
     +      'se(sigma)',
     +      '       xi',
     +      '   se(xi)',
     +      '       xp',
     +      '   se(xp)'
      print *
      print '(a23,1x,i5,1x,f5.3,9(1x,f9.3))',
     +      trim(adjustl(infile)),m,p,1.0/p,
     +      mu,sqrt(varmu),sigma,sqrt(varsigma),
     +      xi,sqrt(varxi),xp,sqrt(varxp)
      
      print *
      print *
      print *
      print *, ' GEVMLEST terminated.'
      print *, ' ==================='
      print *
      stop
      end
c      
c=======================================================================
c      
      SUBROUTINE MLEGEV(X, N, PARA, VCOV, MONIT, IFAULT)

C        ALGORITHM AS215   APPL. STATIST. (1985) VOL. 34, NO. 3
C        Modifications in AS R76 (1989) have been incorporated.
C
C        Additional modifications by J. R. M. Hosking, August 1994:
C        Modify steepest-ascent step so that it is invariant to
C        rescaling the data.  Improves chance of convergence from
C        poor initial values.
C
C        MAXIMUM-LIKELIHOOD ESTIMATION OF GENERALIZED EXTREME-VALUE
C        DISTRIBUTION
C
      DOUBLE PRECISION PARA(3), VCOV(6), X(N)
      DOUBLE PRECISION A, ACCA, ACCG, ACCU, AI, AIGI, AN, D, DA, DAA,
     *  DAG, DELA, DELG, DELU, DG, DGG, DU, DUA, DUG, DUU, E, F, FOLD,
     *  G, GAI, GG, GI, GIPQ, GNORM, H, HALF, HE, HH, ONE, P, PA, PQ,
     *  PQG, PU, Q, QA, QU, R, RA, RATIO, RG, RU, SE, SH, SHE, SHH,
     *  SHHE, SMALL, SRF, STEPA, STEPG, STEPU, SY, SYE, SYHE, SYYE,
     *  TEMP1, TEMP2, U, VLNEG, XMAX, XMIN, Y, YE, Z, ZERO
      CHARACTER*8 ACTI1, ACTI2, ACTI3, ACTI4, ACTI5, ACTI6, ACTI7,
     *  ACTI8, ACTI9
      DATA ACTI1/'  NEWTON'/, ACTI2/'  ST.ASC'/, ACTI3/'  RESETK'/,
     *  ACTI4/'  SR.INF'/, ACTI5/'  SR.LIK'/, ACTI6/'  MAX.SR'/,
     *  ACTI7/'  MAX.EV'/, ACTI8/'  MAX.IT'/, ACTI9/'  CONVGD'/
      DATA ZERO /0.0D0/, HALF /0.5D0/, ONE/1.0D0/
C
C        ADDU,ACCA,ACCG ARE ACCURACY CRITERIA FOR TESTING CONVERGENCE
C        STEPU,STEPA,STEPG ARE MAXIMUM STEPLENGTHS FOR ITERATIONS
C        ACCU,ACCA,STEPU,STEPA ARE SCALED BY CURRENT VALUE OF A WHEN
C        USED IN PROGRAM
C
      DATA ACCU, ACCA, ACCG /3 * 1D-5/, STEPU, STEPA, STEPG /
     *  0.5D0, 0.25D0, 0.2D0/
C
C        MAXIT IS MAX. NO. OF ITERATIONS
C        MAXEV IS MAX. NO. OF EVALUATIONS OF LIKELIHOOD FUNCTION
C        SRF IS STEPLENGTH REDUCTION FACTOR
C        MAXSR IS MAX. NO. OF STEPLENGTH REDUCTIONS PERMITTED PER
C        ITERATION
C
      DATA MAXIT /30/, MAXEV /50/, SRF /0.25D0/, MAXSR /30/
C
C        SMALL IS A SMALL NUMBER, USED TO ADJUST THE SHAPE PARAMETER TO
C        AVOID AN EXACT ZERO VALUE OR BORDERLINE INFEASIBILITY
C        ALNEG IS A LARGE NEGATIVE NUMBER, USED TO INITIALIZE
C        LOG-LIKELIHOOD
C
      DATA SMALL /1.D-3/, VLNEG /-1.D37/
C
C        FIND MIN AND MAX DATA VALUE
C
      DO 10 I = 1, 6
  10  VCOV(I) = ZERO
      IFAULT = 1
      IF (N .LE. 2) GOTO 170
      XMIN = X(1)
      XMAX = X(1)
      DO 20 I = 2, N
        IF (X(I) .LT. XMIN) XMIN = X(I)
        IF (X(I) .GT. XMAX) XMAX = X(I)
   20 CONTINUE
C
C       INITIALIZATION
C       U IS LOCATION PARAMETER
C       A IS SCALE PARAMETER
C       G IS SHAPE PARAMETER
C
      IF (MONIT .GT. 0) WRITE (6, 6000)
      IFAULT = 0
      NITER = 0
      NEVAL =0
      FOLD = VLNEG
      U = PARA(1)
      A = PARA(2)
      G = PARA(3)
      IF (ABS(G) .LT. SMALL) G = SMALL
      IF (A .LE. ZERO) A = ONE
      AN = DBLE(FLOAT(N))
C
C        CHECK WHETHER ALL DATA POINTS LIE WITHIN THE RANGE OF THE GEV
C        DISTRIBUTION WITH THE INITIAL PARAMETERS - IF NOT, ADJUST THE
C        SHAPE PARAMETER SO AS TO BRING ALL POINTS WITHIN RANGE
C
      IF (G .GT. ZERO) GOTO 30
      IF (XMIN .GE. U) GOTO 40
      Z = A / (XMIN - U)
      IF (G .GT. Z) GOTO 40
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010) 
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI3
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI3
      G = Z + SMALL
      IF (G .GE. ZERO) G = HALF * Z
      GOTO 40
   30 IF (XMAX .LE. U) GOTO 40
      Z = A / (XMAX - U)
      IF (G .LT. Z) GOTO 40
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI3
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI3
      G = Z - SMALL
      IF (G .LE. ZERO) G = HALF * Z
C
C        START OF MAIN LOOP
C
   40 DO 140 NITER = 1, MAXIT
C
      NSR = 0
   50 IF (NEVAL .GE. MAXEV) GOTO 150
      NEVAL = NEVAL + 1
      AI = ONE / A
      GI = ONE /G
      GAI = G * AI
      AIGI = AI * GI
      GG = ONE - G
C
C       ACCUMULATE SUMS OF QUANTITIES OCCURRING IN LIKELIHOOD
C       DERIVATIVES
C
C       IN PRESCOTT AND WALDEN'S NOTATION:
C       Z IS 1 - K * (X(I)-U) / A
C       Y IS THE REDUCED VARIATE - (1/K) * LOG(Z)
C       E IS EXP(-Y)
C       H IS EXP(K*Y)
C
      SY = ZERO
      SE = ZERO
      SYE = ZERO
      SYYE = ZERO
      SH = ZERO
      SHE = ZERO
      SYHE = ZERO
      SHHE = ZERO
      SHH = ZERO
      DO 60 I = 1, N
        Z = ONE -GAI * (X(I) - U)
        Y = -GI * LOG(Z)
        E = EXP(-Y)
        H = ONE / Z
        YE = Y * E
        HE = H * E
        HH = H * H
        SY = SY + Y
        SE = SE + E
        SYE = SYE + YE
        SYYE = SYYE + Y * YE
        SH = SH + H
        SHE = SHE + HE
        SYHE = SYHE + Y * HE
        SHHE = SHHE + HH * E
        SHH = SHH + HH
   60 CONTINUE
C
C       F IS CURRENT VALUE OF LIKELIHOOD FUNCTIONN
C
      F = -AN * LOG(A) - GG * SY - SE
      IF (F .GT. FOLD) GOTO 90
C
C       LIKELIHOOD HAS NOT INCREASED - REDUCE STEPLENGTH AND TRY AGAIN
C
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI5, F
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI5, F
      IF (NSR .EQ. MAXSR) GOTO 80
   70 NSR = NSR + 1
      U = U - DELU
      A = A - DELA
      G = G - DELG
      DELU = SRF * DELU
      DELA = SRF * DELA
      DELG = SRF * DELG
      U = U + DELU
      A = A + DELA
      G = G + DELG
      IF (A .GT. G * (XMIN - U) .AND. A .GT. G * (XMAX - U) .AND. G .NE.
     *  ZERO) GO TO 50
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI4
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI4
      IF (NSR .LT. MAXSR) GOTO 70
C
C        MAX. NO. OF STEPLENGTH REDUCTIONS REACHED
C        IF CURRENT ITERATION IS NEWTON-RAPHSON, TRY STEEPEST ASCENT
C        INSTEAD.  IF CURRENT ITERATION IS STEEPEST ASCENT, GIVE UP.
C
   80 U = U - DELU
      A = A - DELA
      G = G - DELG
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI6
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI6
      IF (ITYPE .EQ. 1) GOTO 100
      IFAULT = 4
      GOTO 160
C
C        P,Q,R, ARE AS DEFINED IN FLOOD STUDIES REPORT
C
   90 FOLD = F
      P = AN - SE
      Q = SHE - GG * SH
      R = AN - SY + SYE
      PQ = P + Q
      GIPQ = GI * PQ
C
C        FIRST DERIVATIVES OF LOG-LIKELIHOOD
C
      DU = -AI * Q
      DA = -AIGI * PQ
      DG = -GI * (R - GIPQ)
      IF (MONIT .GT. 0) GNORM = SQRT(DU * DU + DA * DA + DG * DG)
C
C        DERIVATIVES OF P,Q,R
C
      PU = -AI * SHE
      PA = GI * PU + AIGI * SE
      QU = GG * AI * (SHHE + G * SHH)
      RU = AI * (SH - SHE + SYHE)
      RA = GI * RU - AIGI * (AN - SE + SYE)
      RG = GI * (SY - SYE + SYYE - A * RA)
      QA = AI * Q + GI * (PU + QU)
      PQG = GIPQ + A * (RA - GI * (PA + QA))
C
C         MINUS SECOND DERIVATIVE OF LOG-LIKELIHOOD (HESSIAN MATRIX)
C
      DUU = AI * QU
      DUA = AIGI * (PU + QU)
      DAA = -AIGI * (AI * PQ - PA - QA)
      DUG = GI * (RU - GI * (PU + QU))
      DAG = -AIGI * (GIPQ - PQG)
      DGG = GI * (RG - GI * (PQG + R - GIPQ - GIPQ))
C
C         INVERT HESSIAN MATRIX
C
      DO 95 KK = 1, 3
        IF (DUU .LE. ZERO) GO TO 100
        D = ONE / DUU
        TEMP1 = -DUA * D
        IF (KK .GT. 2) TEMP1 = -TEMP1
        TEMP2 = -DUG * D
        IF (KK .GT. 1) TEMP2 = -TEMP2
        DUU = DAA + TEMP1 * DUA
        DUA = DAG + TEMP1 * DUG
        DAA = DGG + TEMP2 * DUG
        DUG = TEMP1
        DAG = TEMP2
        DGG = D
   95 CONTINUE
C
C        CALCULATE STEPLENGTHS
C
      ITYPE = 1
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG),
     +ACTI1, F, GNORM
c     IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI1, F,
c     *  GNORM
      DELU = DUU * DU + DUA * DA + DUG * DG
      DELA = DUA * DU + DAA * DA + DAG * DG
      DELG = DUG * DU + DAG * DA + DGG * DG
      RATIO = MAX(ABS(DELU) / (STEPU * A), ABS(DELA) / (STEPA * A),
     *  ABS(DELG) / STEPG)
      IF (RATIO .LT. ONE) GOTO 110
      RATIO = ONE / RATIO
      DELU = DELU * RATIO
      DELA = DELA * RATIO
      DELG = DELG * RATIO
      GOTO 110
C
C        HESSIAN IS NOT POSITIVE DEFINITE - MAKE A LARGE STEP IN THE
C        DIRECTION OF STEEPEST ASCENT
C
  100 ITYPE = 2
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG),
     +ACTI2, F, GNORM
c     IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI2, F,
c     *  GNORM
C---------------------------------------  NEXT 11 LINES ADDED, AUG.94
      D = ABS(VLNEG)
      TEMP1 = D
      IF (DU .NE. ZERO) TEMP1 = STEPU / (ABS(DU) * A)
      TEMP2 = D
      IF (DA .NE. ZERO) TEMP2 = STEPA / (ABS(DA) * A)
      Z = D
      IF (DG .NE. ZERO) Z = STEPG / ABS(DG)
      RATIO = MIN(TEMP1, TEMP2, Z)
      DELU = RATIO * DU * A * A
      DELA = RATIO * DA * A * A
      DELG = RATIO * DG
C---------------------------------------  NEXT 11 LINES REMOVED, AUG.94
C      D = ABS(VLNEG)
C      TEMP1 = D
C      IF (DU .NE. ZERO) TEMP1 = STEPU * A / ABS(DU)
C      TEMP2 = D
C      IF (DA .NE. ZERO) TEMP2 = STEPA * A / ABS(DA)
C      Z = D
C      IF (DG .NE. ZERO) Z = STEPG / ABS(DG)
C      RATIO = MIN(TEMP1, TEMP2, Z)
C      DELU = RATIO * DU
C      DELA = RATIO * DA
C      DELG = RATIO * DG
C---------------------------------------  END OF DELETED CODE
C
C        ADJUST PARAMETERS
C
  110 U = U + DELU
      A = A + DELA
      G = G + DELG
C
C        TEST FOR FEASIBILITY
C
      IF (A .GT. G * (XMIN - U) .AND. A .GT. G * (XMAX - U) .AND. G .NE.
     *  ZERO) GOTO 130
      DO 120 NSR = 1, MAXSR
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
        IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI4
c        IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI4
        U = U - DELU
        A = A - DELA
        G = G - DELG
        DELU = SRF * DELU
        DELA = SRF * DELA
        DELG = SRF * DELG
        U = U + DELU
        A = A + DELA
        G = G + DELG
        IF (A .GT. G * (XMIN - U) .AND. A .GT. G * (XMAX - Y) .AND.
     *     G .NE. ZERO) GOTO 140
  120 CONTINUE
      GOTO 80
C
C        TEST FOR CONVERGENCE
C
  130 IF (ABS(DELU) .GT. ACCU * A) GOTO 140
      IF (ABS(DELA) .GT. ACCA * A) GOTO 140
      IF (ABS(DELG) .GT. ACCG) GOTO 140
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI9      
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, NEVAL, U, A, G, ACTI9
      VCOV(1) = DUU
      VCOV(2) = DUA
      VCOV(3) = DAA
      VCOV(4) = DUG
      VCOV(5) = DAG
      VCOV(6) = DGG
      GOTO 160
C
C        END OF MAIN LOOP
C
  140 CONTINUE
C
C        ITERATIONS NOT CONVERGED - SET FAULT FLAG
C
      IFAULT = 2
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI8      
c      IF (MONIT .GT. 0) WRITE (6, 6010) MAXIT, NEVAL, U, A, G, ACTI8
      GOTO 160
  150 IFAULT = 3
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
      IF (MONIT .GT. 0) WRITE (6,6010)
     +NITER, NEVAL, U, SQRT(DUU), A,SQRT( DAA), -G, SQRT(DGG), ACTI7  
c      IF (MONIT .GT. 0) WRITE (6, 6010) NITER, MAXEV, U, A, G, ACTI7
C
C        ITERATION FINISHED -COPY RESULTS INTO ARRAY PARA
C
  160 IF (MONIT .GT. 0) WRITE(6, 6020)
      PARA(1) = U
      PARA(2) = A
      PARA(3) = G
      RETURN
C
  170 DO 180 I = 1, 3
  180 PARA(I) = ZERO
      RETURN
c
c Changed (Mudelsee: Parameter notation according to Coles (2001).)
c
 6000 FORMAT(/' MAXIMUM-LIKELIHOOD ESTIMATION OF GENERALIZED EXTREME',
     *  1X, 'VALUE DISTRIBUTION'// ' ITER EVAL', 8X,
     + 'mu    se(mu)', 5X, 'sigma se(sigma)',
     *  8X, 'xi    se(xi)   ACTION', 6X, 'LOG-L', 7X, 'GNORM')
c
c 6000 FORMAT(/' MAXIMUM-LIKELIHOOD ESTIMATION OF GENERALIZED EXTREME',
c     *  1X, 'VALUE DISTRIBUTION'// ' ITER EVAL', 8X, 'XI', 5X, 'ALPHA',
c     *  9X, 'K  ACTION', 6X, 'LOG-L', 7X, 'GNORM')
 6010 FORMAT(1X, I4, I5, 6F10.4, 1X, A, F11.3, 1PD12.2)
 6020 FORMAT(1H0)
      END